﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Globalization;

namespace ClassLibrary1
{
    public class Class1 : Component
    {
        private Size _size;

        public Class1()
        {
            _size = new Size();
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        [TypeConverter(typeof(SizeConverter))]       //  —— 注1，也可以把这句TypeConverterAttribute写在注2处。
        public Size Size
        {
            get { return _size; }
            set { _size = value; }
        }
    }

    public class SizeConverter : TypeConverter          // 我们自定义的Converter必须继承于TypeConverter基类。
    {
        /// <summary>
        /// 是否能用string转换到Size类型。
        /// </summary>
        /// <param name="context">上下文。</param>
        /// <param name="sourceType">转换源的Type。</param>
        /// <returns></returns>
        public override bool CanConvertFrom(ITypeDescriptorContext context, Type sourceType)
        {
            if (sourceType == typeof(string))
            { return true; }
            else
            { return false; }
        }

        /// <summary>
        /// 从string转到Size类型。
        /// </summary>
        /// <param name="context">提供Component的上下文，如Component.Instance对象等。</param>
        /// <param name="culture">提供区域信息，如语言、时间格式、货币格式等</param>
        /// <param name="value"></param>
        /// <returns></returns>
        public override object ConvertFrom(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value)
        {
            if (value == null || value.ToString().Length == 0) return new Size();
            char spliter = culture.TextInfo.ListSeparator[0];    // 得到字符串的分隔符
            string[] ss = ((string)value).Split(spliter);

            Int32Converter intConverter = new Int32Converter();   // 得到类型转换器，.net中为我们定义了一些常见的类型转换器。
            return new Size((Int32)intConverter.ConvertFromString(context, culture, ss[0]),  
               (Int32)intConverter.ConvertFromString(context, culture, ss[1]));
        }

        /// <summary>
        /// 是否能用Size转换到string类型。
        /// </summary>
        /// <param name="context"></param>
        /// <param name="destinationType">转换目标的类型。</param>
        /// <returns></returns>
        public override bool CanConvertTo(ITypeDescriptorContext context, Type destinationType)
        {
            if (destinationType == typeof(Size))  // 如果是Size格式，则允许转成string。
            { return true; }
            else
            { return false; }
        }

        // 在Property窗口中显示为string类型。
        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, Type destinationType)
        {
            if (value == null) return string.Empty;
            if (destinationType == typeof(string))
            {
                Size size = (Size)value;
                TypeConverter intConverter = TypeDescriptor.GetConverter(typeof(Int32));   // 能到类型转换器的另一种方式。
                char spliter = culture.TextInfo.ListSeparator[0];    // 得到字符串的分隔符

                return string.Join(spliter.ToString(), new string[]{
                intConverter.ConvertToString(context, culture, size.Length),
                    intConverter.ConvertToString(context, culture, size.Width)});
            }
            return string.Empty;
        }

        // TypeConverter还有几个虚方法，请大家自己研究。
    }

    // [TypeConverter(typeof(SizeConverter))]     —— 注2
    public class Size
    {
        private Int32 _length;
        private Int32 _width;

        public Size(Int32 length, Int32 width)
        {
            _length = length;
            _width = width;
        }

        public Size() : this(0, 0)
        {
        }

        public Int32 Length
        {
            get { return _length; }
            set { _length = value; }
        }

        public Int32 Width
        {
            get { return _width; }
            set { _width = value; }
        }
    }
}
